@@ -24,8 +24,8 @@
 #include <stdio.h>
 #include <stdlib.h>
 
-static GCond *run_cond;
-static GMutex *run_lock;
+static GCond run_cond;
+static GMutex run_lock;
 static struct coroutine *current;
 static struct coroutine leader;
 
@@ -37,15 +37,18 @@ static struct coroutine leader;
 
 static void coroutine_system_init(void)
 {
-    run_cond = g_cond_new();
-    run_lock = g_mutex_new();
-    CO_DEBUG("LOCK");
-    g_mutex_lock(run_lock);
+    if (current != NULL) {
+        return;
+    }
 
-    /* The thread that creates the first coroutine is the system coroutine
-     * so let's fill out a structure for it */
+    CO_DEBUG("INIT");
+
+    CO_DEBUG("LOCK");
+    g_mutex_lock(&run_lock);
+
+    /* The thread that creates the first coroutine is the system
+     * coroutine so let's fill out a structure for it */
     leader.entry = NULL;
-    leader.release = NULL;
     leader.stack_size = 0;
     leader.exited = 0;
     leader.thread = g_thread_self();
@@ -60,44 +63,37 @@ static gpointer coroutine_thread(gpointer opaque)
 {
     struct coroutine *co = opaque;
     CO_DEBUG("LOCK");
-    g_mutex_lock(run_lock);
+    g_mutex_lock(&run_lock);
     while (!co->runnable) {
         CO_DEBUG("WAIT");
-        g_cond_wait(run_cond, run_lock);
+        g_cond_wait(&run_cond, &run_lock);
     }
 
     CO_DEBUG("RUNNABLE");
     current = co;
-    co->data = co->entry(co->data);
+    co->caller->data = co->entry(co->data);
     co->exited = 1;
 
     co->caller->runnable = TRUE;
+    co->caller = NULL;
     CO_DEBUG("BROADCAST");
-    g_cond_broadcast(run_cond);
+    g_cond_broadcast(&run_cond);
     CO_DEBUG("UNLOCK");
-    g_mutex_unlock(run_lock);
+    g_mutex_unlock(&run_lock);
 
     return NULL;
 }
 
-int coroutine_init(struct coroutine *co)
+void coroutine_init(struct coroutine *co)
 {
-    if (run_cond == NULL)
-        coroutine_system_init();
-
+    coroutine_system_init();
     CO_DEBUG("NEW");
-    co->thread = g_thread_create_full(coroutine_thread, co, co->stack_size,
-                                      FALSE, TRUE,
-                                      G_THREAD_PRIORITY_NORMAL,
-                                      NULL);
-    if (co->thread == NULL)
-        return -1;
+    co->thread = g_thread_new("coroutine-thread",
+                              coroutine_thread, co);
 
     co->exited = 0;
     co->runnable = FALSE;
     co->caller = NULL;
-
-    return 0;
 }
 
 int coroutine_release(struct coroutine *co G_GNUC_UNUSED)
@@ -105,21 +101,20 @@ int coroutine_release(struct coroutine *co G_GNUC_UNUSED)
     return 0;
 }
 
-void *coroutine_swap(struct coroutine *from, struct coroutine *to, void *arg)
+static void *coroutine_swap(struct coroutine *from, struct coroutine *to, void *arg)
 {
     from->runnable = FALSE;
     to->runnable = TRUE;
     to->data = arg;
-    to->caller = from;
     CO_DEBUG("BROADCAST");
-    g_cond_broadcast(run_cond);
+    g_cond_broadcast(&run_cond);
     CO_DEBUG("UNLOCK");
-    g_mutex_unlock(run_lock);
+    g_mutex_unlock(&run_lock);
     CO_DEBUG("LOCK");
-    g_mutex_lock(run_lock);
+    g_mutex_lock(&run_lock);
     while (!from->runnable) {
         CO_DEBUG("WAIT");
-        g_cond_wait(run_cond, run_lock);
+        g_cond_wait(&run_cond, &run_lock);
     }
     current = from;
 
@@ -129,16 +124,17 @@ void *coroutine_swap(struct coroutine *from, struct coroutine *to, void *arg)
 
 struct coroutine *coroutine_self(void)
 {
+    coroutine_system_init();
     return current;
 }
 
 void *coroutine_yieldto(struct coroutine *to, void *arg)
 {
-    if (to->caller) {
-        fprintf(stderr, "Co-routine is re-entering itself\n");
-        abort();
-    }
+    g_return_val_if_fail(!to->caller, NULL);
+    g_return_val_if_fail(!to->exited, NULL);
+
     CO_DEBUG("SWAP");
+    to->caller = coroutine_self();
     return coroutine_swap(coroutine_self(), to, arg);
 }
 
