@@ -25,6 +25,9 @@
 #include <sys/mman.h>
 #include <stdio.h>
 #include <stdlib.h>
+#ifdef HAVE_VALGRIND
+# include <valgrind/valgrind.h>
+#endif
 #include "coroutine.h"
 
 /* Back compat for macOS 10.x https://gitlab.gnome.org/GNOME/gtk-vnc/issues/4 */
@@ -41,12 +44,9 @@ static int _coroutine_release(struct continuation *cc)
 {
     struct coroutine *co = container_of(cc, struct coroutine, cc);
 
-    if (co->release) {
-        int ret = co->release(co);
-        if (ret < 0)
-            return ret;
-    }
-
+#ifdef HAVE_VALGRIND
+    VALGRIND_STACK_DEREGISTER(co->vg_stack);
+#endif
     munmap(co->cc.stack, co->cc.stack_size);
 
     co->caller = NULL;
@@ -60,7 +60,7 @@ static void coroutine_trampoline(struct continuation *cc)
     co->data = co->entry(co->data);
 }
 
-int coroutine_init(struct coroutine *co)
+void coroutine_init(struct coroutine *co)
 {
     if (co->stack_size == 0)
         co->stack_size = 16 << 20;
@@ -71,31 +71,29 @@ int coroutine_init(struct coroutine *co)
                         MAP_PRIVATE | MAP_ANONYMOUS,
                         -1, 0);
     if (co->cc.stack == MAP_FAILED)
-        g_error("Failed to allocate %u bytes for coroutine stack",
-                (unsigned)co->stack_size);
+        g_error("mmap(%" G_GSIZE_FORMAT ") failed: %s",
+                co->stack_size, g_strerror(errno));
+
+#ifdef HAVE_VALGRIND
+    co->vg_stack = VALGRIND_STACK_REGISTER(co->cc.stack, co->cc.stack + co->stack_size);
+#endif
+
     co->cc.entry = coroutine_trampoline;
     co->cc.release = _coroutine_release;
     co->exited = 0;
 
-    return cc_init(&co->cc);
+    cc_init(&co->cc);
 }
 
-#if 0
-static __thread struct coroutine leader;
-static __thread struct coroutine *current;
-#else
 static struct coroutine leader;
-static struct coroutine *current;
-#endif
+static struct coroutine *current = &leader;
 
 struct coroutine *coroutine_self(void)
 {
-    if (current == NULL)
-        current = &leader;
     return current;
 }
 
-void *coroutine_swap(struct coroutine *from, struct coroutine *to, void *arg)
+static void *coroutine_swap(struct coroutine *from, struct coroutine *to, void *arg)
 {
     int ret;
     to->data = arg;
@@ -105,7 +103,7 @@ void *coroutine_swap(struct coroutine *from, struct coroutine *to, void *arg)
         return from->data;
     else if (ret == 1) {
         coroutine_release(to);
-        current = &leader;
+        current = from;
         to->exited = 1;
         return to->data;
     }
@@ -115,10 +113,9 @@ void *coroutine_swap(struct coroutine *from, struct coroutine *to, void *arg)
 
 void *coroutine_yieldto(struct coroutine *to, void *arg)
 {
-    if (to->caller) {
-        fprintf(stderr, "Co-routine is re-entering itself\n");
-        abort();
-    }
+    g_return_val_if_fail(!to->caller, NULL);
+    g_return_val_if_fail(!to->exited, NULL);
+
     to->caller = coroutine_self();
     return coroutine_swap(coroutine_self(), to, arg);
 }
@@ -133,10 +130,3 @@ void *coroutine_yield(void *arg)
     coroutine_self()->caller = NULL;
     return coroutine_swap(coroutine_self(), to, arg);
 }
-/*
- * Local variables:
- *  c-indent-level: 4
- *  c-basic-offset: 4
- *  indent-tabs-mode: nil
- * End:
- */
