@@ -819,6 +819,10 @@ static void vnc_base_framebuffer_reinit_render_funcs(VncBaseFramebuffer *fb)
     else
         priv->perfect_match = FALSE;
 
+    VNC_DEBUG("Byte order local: %d remote: %d native: %d\n",
+              priv->localFormat->byte_order,
+              priv->remoteFormat->byte_order,
+              G_BYTE_ORDER);
     depth = priv->remoteFormat->depth;
     if (depth == 32)
         depth = 24;
@@ -826,7 +830,8 @@ static void vnc_base_framebuffer_reinit_render_funcs(VncBaseFramebuffer *fb)
     priv->rm = priv->localFormat->red_max & priv->remoteFormat->red_max;
     priv->gm = priv->localFormat->green_max & priv->remoteFormat->green_max;
     priv->bm = priv->localFormat->blue_max & priv->remoteFormat->blue_max;
-    VNC_DEBUG("Mask local: %3d %3d %3d\n"
+    VNC_DEBUG("Mask\n"
+              "    local: %3d %3d %3d\n"
               "    remote: %3d %3d %3d\n"
               "    merged: %3d %3d %3d",
               priv->localFormat->red_max, priv->localFormat->green_max, priv->localFormat->blue_max,
@@ -841,6 +846,11 @@ static void vnc_base_framebuffer_reinit_render_funcs(VncBaseFramebuffer *fb)
     priv->rls = priv->localFormat->red_shift;
     priv->gls = priv->localFormat->green_shift;
     priv->bls = priv->localFormat->blue_shift;
+    VNC_DEBUG("Original pixel shifts\n"
+              "    remote: R=%-3d G=%-3d B=%-3d\n"
+              "     local: R=%-3d G=%-3d B=%-3d",
+              priv->rrs, priv->grs, priv->brs,
+              priv->rls, priv->gls, priv->bls);
 
     /* This adjusts for remote having more bpp than local */
     for (n = priv->remoteFormat->red_max; n > priv->localFormat->red_max ; n>>= 1)
@@ -857,7 +867,10 @@ static void vnc_base_framebuffer_reinit_render_funcs(VncBaseFramebuffer *fb)
         priv->gls++;
     for (n = priv->localFormat->blue_max ; n > priv->remoteFormat->blue_max ; n>>= 1)
         priv->bls++;
-    VNC_DEBUG("Pixel shifts\n   right: %3d %3d %3d\n    left: %3d %3d %3d",
+
+    VNC_DEBUG("Adjusted pixel shifts\n"
+              "    remote: R=%-3d G=%-3d B=%-3d\n"
+              "     local: R=%-3d G=%-3d B=%-3d",
               priv->rrs, priv->grs, priv->brs,
               priv->rls, priv->gls, priv->bls);
 
