@@ -310,3 +310,25 @@ guint16 vnc_cursor_get_height(VncCursor *cursor)
 
     return priv->height;
 }
+
+
+/**
+ * vnc_cursor_is_visible:
+ * @cursor: the cursor object
+ *
+ * Determine whether the rendered cursor has
+ * any visible pixels
+ *
+ * Returns: TRUE if there is at least one visible pixel, FALSE otherwise
+ */
+gboolean vnc_cursor_is_visible(VncCursor *cursor)
+{
+    VncCursorPrivate *priv = cursor->priv;
+    int i;
+    gsize len = priv->width * priv->height * 4;
+    for (i = 0; i < len; i++) {
+        if (priv->data[i])
+            return TRUE;
+    }
+    return FALSE;
+}
